package router

import (
	"context"
	"encoding/json"
	"fmt"
	"gibb165lb2/util"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"log"
	"net/http"
)

// GetSpiele curl -X GET localhost:3000/spiele
func GetSpiele(w http.ResponseWriter, r *http.Request) {
	// MongoDB-Datenbank und -Sammlung auswählen
	db := util.Client.Database("meineDatenbank")
	spieleCollection := db.Collection("spiele")

	// Spiele aus der Datenbank abrufen
	var spiele []util.Spiel
	cursor, err := spieleCollection.Find(context.Background(), bson.M{})
	if err != nil {
		log.Fatal(err)
	}
	defer func(cursor *mongo.Cursor, ctx context.Context) {
		err := cursor.Close(ctx)
		if err != nil {

		}
	}(cursor, context.Background())
	for cursor.Next(context.Background()) {
		var spiel util.Spiel
		if err := cursor.Decode(&spiel); err != nil {
			log.Fatal(err)
		}
		spiele = append(spiele, spiel)
	}
	if err := cursor.Err(); err != nil {
		log.Fatal(err)
	}

	// Spiele als JSON zurücksenden
	w.Header().Set("Content-Type", "application/json")
	if err := json.NewEncoder(w).Encode(spiele); err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		fmt.Println(err)
	}
}

// DeleteSpiel curl -X DELETE -d '{"spiel_id": "YOUR_SPIEL_ID"}' -H "Content-Type: application/json" localhost:3000/deleteSpiel
func DeleteSpiel(w http.ResponseWriter, r *http.Request) {
	// Parse JSON body
	var requestData struct {
		SpielID string `json:"spiel_id"`
	}
	err := json.NewDecoder(r.Body).Decode(&requestData)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// Parse SpielID as ObjectID
	spielID, err := primitive.ObjectIDFromHex(requestData.SpielID)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// MongoDB-Datenbank und -Sammlungen auswählen
	db := util.Client.Database("meineDatenbank")
	spieleCollection := db.Collection("spiele")

	// Spiel aus der Datenbank löschen
	_, err = spieleCollection.DeleteOne(context.Background(), bson.M{"_id": spielID})
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	// Erfolgsmeldung senden
	w.WriteHeader(http.StatusOK)
	_, err = w.Write([]byte("Spiel erfolgreich gelöscht"))
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
}

// GetSpieleMitLaeden curl -X GET localhost:3000/spieleMitLaeden
func GetSpieleMitLaeden(w http.ResponseWriter, r *http.Request) {
	// MongoDB-Datenbank und -Sammlungen auswählen
	db := util.Client.Database("meineDatenbank")
	spieleCollection := db.Collection("spiele")
	laedenCollection := db.Collection("spielelaeden")

	// Spiele aus der Datenbank abrufen
	var spieleMitLaeden []util.SpieleMitLaden
	cursor, err := spieleCollection.Find(context.Background(), bson.M{})
	if err != nil {
		log.Fatal(err)
	}
	defer func(cursor *mongo.Cursor, ctx context.Context) {
		err := cursor.Close(ctx)
		if err != nil {

		}
	}(cursor, context.Background())
	for cursor.Next(context.Background()) {
		var spiel util.Spiel
		if err := cursor.Decode(&spiel); err != nil {
			log.Fatal(err)
		}
		spieleMitLaedenMut := spiel.ToSpieleMitLaden()
		fmt.Printf("%+v\n", spieleMitLaedenMut)
		// Lädeninformationen für jedes Spiel abrufen
		for _, ladenID := range spiel.Laeden {
			var laden util.Spieleladen
			err := laedenCollection.FindOne(context.Background(), bson.M{"_id": ladenID}).Decode(&laden)
			if err != nil {
				log.Fatal(err)
			}
			spieleMitLaedenMut.Laeden = append(spieleMitLaedenMut.Laeden, laden)
		}
		spieleMitLaeden = append(spieleMitLaeden, spieleMitLaedenMut)
	}
	if err := cursor.Err(); err != nil {
		log.Fatal(err)
	}

	// Spiele mit Läden als JSON zurücksenden
	w.Header().Set("Content-Type", "application/json")
	err = json.NewEncoder(w).Encode(spieleMitLaeden)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
}

// AddNewSpiel curl -X POST -d '{"titel": "Neues Spiel", "erscheinungsdatum": "2024-04-05T00:00:00Z", "publisher": "Neuer Publisher", "genre": "Neues Genre", "plattform": "Neue Plattform", "bewertung": 9}' -H "Content-Type: application/json" localhost:3000/addNewSpiel
func AddNewSpiel(w http.ResponseWriter, r *http.Request) {
	// Parse JSON body
	var newSpiel util.Spiel
	err := json.NewDecoder(r.Body).Decode(&newSpiel)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// MongoDB-Datenbank und -Sammlung auswählen
	db := util.Client.Database("meineDatenbank")
	spieleCollection := db.Collection("spiele")

	// Spiel in die Datenbank einfügen
	_, err = spieleCollection.InsertOne(context.Background(), newSpiel)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	// Erfolgsmeldung senden
	w.WriteHeader(http.StatusCreated)
	_, err = w.Write([]byte("Neues Spiel erfolgreich hinzugefügt"))
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
}

// AddNewLadenToSpiel curl -X POST -d '{"spiel_id": "YOUR_SPIEL_ID", "neuer_laden_id": "YOUR_NEUER_LADEN_ID"}' -H "Content-Type: application/json" localhost:3000/addNewLadenToSpiel
func AddNewLadenToSpiel(w http.ResponseWriter, r *http.Request) {
	// Parse JSON body
	var requestData struct {
		SpielID string `json:"spiel_id"`
		LadenID string `json:"neuer_laden_id"`
	}
	err := json.NewDecoder(r.Body).Decode(&requestData)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// Parse SpielID as ObjectID
	spielID, err := primitive.ObjectIDFromHex(requestData.SpielID)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// Parse NeuerLadenID as ObjectID
	neuerLadenID, err := primitive.ObjectIDFromHex(requestData.LadenID)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// MongoDB-Datenbank und -Sammlungen auswählen
	db := util.Client.Database("meineDatenbank")
	spieleCollection := db.Collection("spiele")

	// Neuen Laden zur Spiel-Sammlung hinzufügen
	_, err = spieleCollection.UpdateOne(
		context.Background(),
		bson.M{"_id": spielID},
		bson.M{"$push": bson.M{"Läden": neuerLadenID}},
	)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	// Erfolgsmeldung senden
	w.WriteHeader(http.StatusCreated)
	if _, err := w.Write([]byte("Neuer Laden erfolgreich zum Spiel hinzugefügt")); err != nil {
		fmt.Println(err)
		http.Error(w, err.Error(), http.StatusBadRequest)
	}
}

// DeleteLadenFromSpiel curl -X DELETE -d '{"spiel_id": "YOUR_SPIEL_ID", "laden_id": "YOUR_LADEN_ID"}' -H "Content-Type: application/json" localhost:3000/deleteLadenFromSpiel
func DeleteLadenFromSpiel(w http.ResponseWriter, r *http.Request) {
	// Parse JSON body
	var requestData struct {
		SpielID string `json:"spiel_id"`
		LadenID string `json:"laden_id"`
	}
	err := json.NewDecoder(r.Body).Decode(&requestData)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// Parse SpielID as ObjectID
	spielID, err := primitive.ObjectIDFromHex(requestData.SpielID)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	// Parse LadenID as ObjectID
	ladenID, err := primitive.ObjectIDFromHex(requestData.LadenID)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
	// MongoDB-Datenbank und -Sammlungen auswählen
	db := util.Client.Database("meineDatenbank")
	spieleCollection := db.Collection("spiele")

	// Laden aus der Spiel-Sammlung entfernen
	_, err = spieleCollection.UpdateOne(
		context.Background(),
		bson.M{"_id": spielID},
		bson.M{"$pull": bson.M{"Läden": ladenID}},
	)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	// Erfolgsmeldung senden
	w.WriteHeader(http.StatusOK)
	if _, err := w.Write([]byte("Laden erfolgreich aus dem Spiel entfernt")); err != nil {
		fmt.Println(err)
		http.Error(w, err.Error(), http.StatusBadRequest)
	}
}
